﻿using CMS;
using CMS.Base;
using CMS.DataEngine;
using CMS.EventLog;
using CMS.Helpers;
using CMS.Membership;
using CMS.SiteProvider;
using System;
using System.Transactions;
using System.Web;
using System.Web.Security;
using System.Xml;

[assembly: RegisterModule(typeof(CustomWIFAuthenticationModule))]

public class CustomWIFAuthenticationModule : Module
{
    public CustomWIFAuthenticationModule()
        : base("CustomWIFAuthentication")
    {
    }

    protected override void OnInit()
    {
        base.OnInit();

        // Assigns a handler to the event
        // This event occurs when users attempt to access a restricted section of Kentico
        SecurityEvents.AuthenticationRequested.Execute += SignIn_Execute;

        // This event occurs when users attempt to log in on the website
        SecurityEvents.Authenticate.Execute += OnAuthentication;
    }

    private void OnAuthentication(object sender, AuthenticationEventArgs e)
    {
        bool wifEnabled = ValidationHelper.GetBoolean(SettingsHelper.AppSettings["CMSEnableWIF"], SettingsKeyInfoProvider.GetBoolValue(SiteContext.CurrentSiteName + ".CMSWIFEnabled"));
        // If using WIF authentication mode
        if (wifEnabled)
        {
            // If user has been created by Kentico
            if (e.User != null && e.User.IsExternal)
            {
                try
                {
                    // Using TS to allow rollback in case of partial error
                    using (TransactionScope ts = new TransactionScope())
                    {
                        // Remove user from site which removes any associated roles
                        if (e.User.IsInSite(SiteContext.CurrentSiteName))
                        {
                            UserInfoProvider.RemoveUserFromSite(e.UserName, SiteContext.CurrentSiteName);
                            UserInfoProvider.SetUserInfo(e.User);
                            EventLogProvider.LogInformation("AAA Custom code", "OnAuthentication", string.Format("User removed from Site '{0}': {1}", SiteContext.CurrentSiteName, e.UserName));
                        }

                        // Add user to site
                        if (!e.User.IsInSite(SiteContext.CurrentSiteName))
                        {
                            UserInfoProvider.AddUserToSite(e.UserName, SiteContext.CurrentSiteName);
                            UserInfoProvider.SetUserInfo(e.User);
                            EventLogProvider.LogInformation("AAA Custom code", "OnAuthentication", string.Format("User added to Site '{0}': {1}", SiteContext.CurrentSiteName, e.UserName));
                        }

                        // Get and decode SAML assertion from Ping POST
                        string strSamlAssertion = System.Web.HttpUtility.HtmlDecode(HttpContext.Current.Request.Form["wresult"]);

                        byte[] utfBytes = System.Text.Encoding.UTF8.GetBytes(strSamlAssertion);
                        string strSamlAssertionDecoded = System.Text.Encoding.UTF8.GetString(utfBytes);

                        EventLogProvider.LogInformation("SAML Assertion", "OnAuthentication", string.Format(strSamlAssertionDecoded));

                        // Load SAML assertion to XmlDocument to retrieve SAML Attributes for role claims
                        XmlDocument xml = new XmlDocument();
                        xml.LoadXml(strSamlAssertionDecoded);

                        XmlNamespaceManager nsmgr = new XmlNamespaceManager(xml.NameTable);
                        nsmgr.AddNamespace("saml", "urn:oasis:names:tc:SAML:1.0:assertion");

                        // Retrieve the list of groups XML 
                        XmlNode node = xml.SelectSingleNode("//saml:Attribute[@AttributeName='Group']", nsmgr);
                        if (node != null)
                        {
                            xml.LoadXml("<root>" + node.InnerXml + "</root>");
                            XmlNodeList roles = xml.SelectNodes("//saml:AttributeValue", nsmgr);

                            EventLogProvider.LogInformation("AAA Custom Node Count", "OnAuthentication", string.Format("Node Count = {0}", roles.Count));

                            if (roles.Count > 0)
                            {
                                string strRole = string.Empty;
                                bool boolIsAdmin = false;
                                bool boolIsEditor = false;

                                foreach (XmlNode role in roles)
                                {
                                    strRole = role.InnerText;
                                    EventLogProvider.LogInformation("AAA Custom code Role", "OnAuthentication", string.Format("Role = {0}", strRole));

                                    // Add user to role 
                                    RoleInfo ri = RoleInfoProvider.GetRoleInfo(strRole, SiteContext.CurrentSiteID);

                                    // If role exists for site
                                    if (ri != null)
                                    {
                                        // Add user to role for site
                                        if (!e.User.IsInRole(strRole, SiteContext.CurrentSiteName))
                                        {
                                            UserInfoProvider.AddUserToRole(e.User.UserID, ri.RoleID);
                                            UserInfoProvider.SetUserInfo(e.User);
                                            EventLogProvider.LogInformation("AAA Custom code Role", "OnAuthentication", string.Format("Add User to Role: {0} => {1}", e.UserName, ri.RoleName));
                                        }
                                    }
                                    else
                                    {
                                        // You may want to comment this line out if there are too many roles not needed in CMS
                                        EventLogProvider.LogInformation("AAA Custom code Role", "OnAuthentication", string.Format("Role '{0}' that user {1} belongs to does not exist in site '{2}'", strRole, e.UserName, SiteContext.CurrentSiteName));
                                    }

                                    // Set flag if it's admin user
                                    //if (strRole.ToLower() == ConfigurationManager.AppSettings["AAAAdminADGroupName"].ToLower())
                                    //{
                                    //    boolIsAdmin = true;
                                    //}

                                    // Set flag if it's editor
                                    //if (strRole.ToLower() == ConfigurationManager.AppSettings["AAAEditorADGroupName"].ToLower())
                                    //{
                                    //    boolIsEditor = true;
                                    //}
                                }

                                // Add editor privilege for eidotors and admin privilege for admin user 
                                if (boolIsAdmin)
                                {
                                    e.User.SetPrivilegeLevel(UserPrivilegeLevelEnum.GlobalAdmin);
                                    EventLogProvider.LogInformation("AAA Custom code Admin", "OnAuthentication", string.Format("'Global Admin' privilege: {0}", e.UserName));
                                }
                                else if (boolIsEditor)
                                {
                                    e.User.SetPrivilegeLevel(UserPrivilegeLevelEnum.Editor);
                                    EventLogProvider.LogInformation("AAA Custom code Editor", "OnAuthentication", string.Format("'Editor' privilege: {0}", e.UserName));
                                }
                                else
                                {
                                    e.User.SetPrivilegeLevel(UserPrivilegeLevelEnum.None);
                                    EventLogProvider.LogInformation("AAA Custom code None", "OnAuthentication", string.Format("'None' privilege: {0}", e.UserName));
                                }

                                UserInfoProvider.SetUserInfo(e.User);
                            }
                        }

                        // Commit changes
                        ts.Complete();
                    }

                    // Redirect to original page requested
                    string lastVisitedPage = CookieHelper.GetValue("LastVistedUrl");

                    if (!string.IsNullOrEmpty(lastVisitedPage))
                    {
                        var ui = e.User;

                        // Set authentication cookie
                        FormsAuthentication.SetAuthCookie(e.UserName, true);

                        // Set context values
                        MembershipContext.AuthenticatedUser = new CurrentUserInfo(ui, true);

                        UserInfoProvider.SetPreferredCultures(ui);

                        EventLogProvider.LogInformation("AAA Custom code", "OnAuthentication Last Visit", string.Format("Redirection to page where autentication request was made: {0}", lastVisitedPage));
                        URLHelper.Redirect(lastVisitedPage);
                    }
                }
                catch (System.Exception ex)
                {
                    EventLogProvider.LogException("AAA Custom code Exception", "OnAuthentication", ex);
                    throw ex;
                }
            }
        }
    }

    // The handler method, which writes the URL, from which the authentication request was made, to the event log
    // You can replace it with your custom code
    private void SignIn_Execute(object sender, AuthenticationRequestEventArgs e)
    {
        CookieHelper.SetValue("LastVistedUrl", RequestContext.RawURL, DateTime.Now.AddMinutes(30));
        EventLogProvider.LogInformation("AAA Custom code Signin", "Signin_Execute", "Requested URL: " + e.RequestedUrl);
    }
}
